@@ -489,20 +489,25 @@ class Base(Controller):
     def ssh(self):
         from ..ext.sshconf import read_ssh_config
         ssh_config_path = os.path.expanduser('~/.ssh/config')
+        from ..ext.sshconf import print_ls
+        
 
         if self.app.pargs.action=='set':
             ssh_config_obj = read_ssh_config(ssh_config_path)
             ssh_config_obj.set(self.app.pargs.host, HostName=self.app.pargs.ip)
             ssh_config_obj.write(ssh_config_path)
-            print(ssh_config_obj.config())
+            print_ls(ssh_config_obj)
+            
 
         elif self.app.pargs.action=='update':
             from ..ext.aws_ec2 import update_ec2_ssh
             profile ='default' # or  self.app.pargs
-            print(update_ec2_ssh(profile))
+            # print(update_ec2_ssh(profile))
+            update_ec2_ssh(profile)
+            ssh_config_obj = read_ssh_config(ssh_config_path)
+            print_ls(ssh_config_obj)
 
         elif self.app.pargs.action=='ls':
-            from ..ext.sshconf import print_ls
             ssh_config_obj = read_ssh_config(ssh_config_path)
             print_ls(ssh_config_obj)
 
