@@ -2,6 +2,9 @@
 Operations to speed up S3 access.
 '''
 import pandas as pd
+from joblib import delayed, Parallel
+import boto3
+# from profilehooks import profile
 import os
 try:
     term_width = os.get_terminal_size().columns
@@ -28,7 +31,6 @@ def get_lastest_file(dongle_id: str):
     Get the lastest file in the bucket.
     TODO: eventually paratemrize the -1 to get any file
     """
-    import boto3
 
     s3_resource = boto3.resource('s3')
     bucket_name = 'creationlabs-raw-data'
@@ -92,8 +94,11 @@ def get_selected_drives(s3, s3_resource, bucket_name, dongle_id):
     # filter out playlist files/folders
     df = df[df.drive_time.str.len() > 16 ]
 
-    latest_drive = df.iloc[0]
-    last_complete_drives = df[df.seg_num!=0]
+    try:
+        latest_drive = df.iloc[0]
+        last_complete_drives = df[df.seg_num!=0]
+    except IndexError:
+        return pd.Series(), pd.Series()
 
     if len(last_complete_drives) > 0:
         last_complete_drive = last_complete_drives.iloc[0]
@@ -102,23 +107,52 @@ def get_selected_drives(s3, s3_resource, bucket_name, dongle_id):
 
     return latest_drive, last_complete_drive
 
-def get_dongle_drives(s3, s3_resource, bucket_name, dongle_id: str, filter: int, latest=False):
-    from tqdm import tqdm 
+def parallel_get_selected_drives(dongle, bucket_name, latest, latest_files):
+    """
+    Get the selected drives for a dongle.
+    """
+    # setting up parameters for parallel processing
+    s3 = boto3.session.Session(region_name='eu-west-1').client('s3')
+    s3_resource = boto3.resource('s3')
+    
+    latest_drive, last_complete_drive = get_selected_drives(s3, s3_resource, bucket_name, dongle)
+    if latest:       latest_files.append(last_complete_drive)
+    else:  latest_files.append(latest_drive)
+    return latest_files
+    
 
+def get_dongle_drives(s3, s3_resource, bucket_name, dongle_id: str, filter: int, latest=False):
+    from pqdm.processes import pqdm
+    from functools import partial
+    # get number of cores in host computer
+    num_cores = os.cpu_count()
+   
     raw_data_buckets = s3.list_objects_v2(Bucket='creationlabs-raw-data',Delimiter='/')
     dongle_ids = [x['Prefix'].split('/')[0] for x in raw_data_buckets['CommonPrefixes']]
+    
     if filter: dongle_ids = [ x for x in dongle_ids if x not in DONGLES_TO_SKIP]
     else: dongle_ids = [ x for x in dongle_ids if x not in DONGLES_TO_SKIP]
 
     latest_files = []
+    
+    # create partial for parallel_get_selected_drives
+    parallel_get_selected_drives_partial = partial(parallel_get_selected_drives, bucket_name=bucket_name,
+                                                   latest=latest, latest_files=latest_files)
 
-    for did in tqdm(sorted(dongle_ids)):
-        latest_drive, last_complete_drive = get_selected_drives(s3, s3_resource, bucket_name, did)
-        # TODO: this does not feel like the last file
-        if latest:       latest_files.append(last_complete_drive)
-        else:  latest_files.append(latest_drive)
-
-    df = pd.DataFrame(latest_files).reset_index()
+    # get the latest file for each dongle
+    parallel_results = pqdm(n_jobs=num_cores, function=parallel_get_selected_drives_partial,
+                            array=dongle_ids)
+    
+    # for did in tqdm(sorted(dongle_ids)):
+    #     latest_drive, last_complete_drive = get_selected_drives(s3, s3_resource, bucket_name, did)
+    #     # TODO: this does not feel like the last file
+    #     if latest:       latest_files.append(last_complete_drive)
+    #     else:  latest_files.append(latest_drive)
+    
+    df = pd.DataFrame(parallel_results).reset_index()
+    # remap a series of rows in df[0] to a dataframe
+    df = df[0].apply(pd.Series).reset_index(drop=True)
+    
     df = df[~df.ext.isna()]
     return df
     
@@ -127,7 +161,6 @@ def get_drives(target, bucket_name: str = 'creationlabs-raw-data', metadata_file
                         show_n: int = 50, filter=0):
     # TODO: profile
     # get all dongle ids
-    import boto3 
     import humanize
 
     pd.set_option('display.max_colwidth',70)
@@ -168,8 +201,6 @@ def pull_latest(dongle_id):
     """
     Download the latest file in the bucket.
     """
-    import boto3
-
     bucket = 'creationlabs-raw-data'
     s3 = boto3.resource('s3')
     obj = s3.Object(bucket, dongle_id)
@@ -181,7 +212,7 @@ def pull_latest(dongle_id):
     print(f"Running {cmd}")
     os.system(cmd)
 
-
+# @profile
 def add_metadata(df: pd.DataFrame, metadata_file_path: str):
     """
     Add metadata to the dataframe.
